package com.me.asteroids.systems;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.math.Vector2;
import com.me.asteroids.EntityFactory;
import com.me.asteroids.Utils;
import com.me.asteroids.components.AccelerationComponent;
import com.me.asteroids.components.PlayerComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.common.ecs.ComponentMapper;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;
import com.me.common.ecs.EntitySystem;

public class PlayerInputSystem extends EntitySystem {

    public Vector2 tmp = new Vector2(0, 1);

    private ComponentMapper<PositionComponent> positionMapper;
    private ComponentMapper<VelocityComponent> velocityMapper;
    private ComponentMapper<AccelerationComponent> accelMapper;

    public PlayerInputSystem(Engine engine) {
        super(
                engine,
                PositionComponent.class,
                VelocityComponent.class,
                AccelerationComponent.class,
                PlayerComponent.class
        );
        positionMapper = engine.getComponentMapper(PositionComponent.class);
        velocityMapper = engine.getComponentMapper(VelocityComponent.class);
        accelMapper = engine.getComponentMapper(AccelerationComponent.class);
    }

    @Override
    public void processEntity(Entity entity, float dt) {
        PositionComponent positionComponent = positionMapper.get(entity);
        VelocityComponent velocityComponent = velocityMapper.get(entity);
        AccelerationComponent accelComponent = accelMapper.get(entity);

        if (Gdx.input.isKeyPressed(Input.Keys.D)) {
            positionComponent.rotation = Utils.rotate(positionComponent.rotation, -300 * dt);
        }

        if (Gdx.input.isKeyPressed(Input.Keys.A)) {
            positionComponent.rotation = Utils.rotate(positionComponent.rotation, 300 * dt);
        }

        Vector2 acceleration = accelComponent.acceleration;
        Vector2 velocity = velocityComponent.velocity;

        if (Gdx.input.isKeyPressed(Input.Keys.W)) {
            acceleration.set(Utils.setUnitVectorAngle(tmp, positionComponent.rotation).scl(500));
        } else {
            if (!velocity.isZero(1f)) {
                acceleration.set(Utils.setUnitVectorAngleRad(tmp, velocity.angleRad()).scl(-100));
            } else {
                acceleration.set(0, 0);
            }
        }

        if (Gdx.input.isKeyJustPressed(Input.Keys.SPACE)) {
            Entity bullet = EntityFactory.createBullet(engine, entity);
            bullet.activate();
        }
    }

}
